
#include <genieArduino.h>

Genie genie;

// RGB LED control demo
// 4D Systems
// This Demo communicates with a 4D Systems Display, configured with ViSi-Genie.
// The display has three sliders and a reset button, each of which was configured 
// to report a message when its status has changed.
// The program simply waits for messages. The messages are received, queued, dequeued, then interpreted.
// The status of a slider controls the brightness of the corresponding LED.
// The reset button moves the slider value and sets the LED brightness level to 128.


  int RED = 9;           // Red LED is connected to pin 9
  int GREEN = 10;        // Green LED is connected to pin 10
  int BLUE = 11;         // Blue LED is connected to pin 11

void setup() 
{ 
  // a few options to talk to the Display, uncomment the preferred serial port and baud rate
  // Make sure that the ViSi Genie program has the same baud rate.
  Serial.begin(9600);
  genie.Begin (Serial);  //Serial0
  genie.AttachEventHandler(myGenieEventHandler);
  
  pinMode(RED, OUTPUT);
  pinMode(GREEN, OUTPUT);
  pinMode(BLUE, OUTPUT);

  //Reset the Display (change D4 to D2 if you have original 4D Arduino Adaptor)
  //See the app note 4D-AN-P4017-ViSi-Genie Connecting a 4D Display to an Arduino Host
  //for information if using jumper connecting wires
  pinMode(4, OUTPUT);  // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(4, 1);  // Reset the Display via D4
  delay(100);
  digitalWrite(4, 0);  // unReset the Display via D4
  
  delay (3500); //let the display start up
}

void loop() 
{ 
  genie.DoEvents();  //Here the messages are received and queued.
}


/////////////////////////////////////////////////////////////////////
//
// This is the user's event handler. It is called by genieDoEvents() 
// when the following conditions are true
//
//		The link is in an IDLE state, and
//		There is an event to handle
//
// See 4D-AN-P4017-ViSi-Genie Connecting a 4D Display to an Arduino Host
// for more information

void myGenieEventHandler(void) 
{
  genieFrame Event;
  genie.DequeueEvent(&Event);

  int slider_val = 0;

  //If the cmd received is from a Reported Event
  if(Event.reportObject.cmd == GENIE_REPORT_EVENT)
  {
    if (Event.reportObject.object == GENIE_OBJ_WINBUTTON)// If the reported message is from a Winbutton
    {
      if (Event.reportObject.index == 0x00)              // If the reported message is from Winbutton0
      genie.WriteObject(GENIE_OBJ_SLIDER, 0x00, 128);     // Set Slider0 value to 128
      genie.WriteObject(GENIE_OBJ_SLIDER, 0x01, 128);     // Set Slider1 value to 128
      genie.WriteObject(GENIE_OBJ_SLIDER, 0x02, 128);     // Set Slider2 value to 128
      analogWrite(RED, 128);                             // Set red LED brightness level to 128
      analogWrite(GREEN, 128);                           // Set green LED brightness level to 128
      analogWrite(BLUE, 128);                            // Set blue LED brightness level to 128      
    }
    
    else if (Event.reportObject.object == GENIE_OBJ_SLIDER)  // If the Reported Message is from a slider
    {
      slider_val = (Event.reportObject.data_msb << 8) + Event.reportObject.data_lsb;  // data of message is stored 
                                                                                      // in variable slider_val
      
      if (Event.reportObject.index == 0x00)                                           // If Reported Message is from Slider0
        analogWrite(RED, slider_val);                                                 // Write value to red LED
      
      else if (Event.reportObject.index == 0x01)                                      // If Reported Message is from Slider1
        analogWrite(GREEN, slider_val);                                               // Write value to green LED
            
      else if (Event.reportObject.index == 0x02)                                      // If Reported Message is from Slider2
        analogWrite(BLUE, slider_val);                                                // Write value to blue LED
    }    
  } 
}

