/*
 * Copyright (c) 2015, Majenko Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * * Neither the name of Majenko Technologies nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// Companion sketch for the UpDownRepeat example in ViSi-Genie Samples. 
// The example needs minor editing for it to work with this sketch. 
// Attached is a slightly modified version of the said sample.
// Requires a PRO license for ViSi-Genie Magic to operate.

#include <genieArduino.h>
#include <SoftwareSerial.h>

#define RESETLINE 4  // Change this if you are not using an Arduino Adaptor Shield Version 2 (see code below)

Genie genie;

// Use software serial on pins 10 and 11, which means set the RX and TX jumpers on
// the Arduino breakout board to use D10 and D11.
SoftwareSerial serial(10, 11);

int counter;

void setup()
{
  Serial.begin(9600);  // Console in Arduino IDE on Serial0
  serial.begin(9600);  
  genie.Begin(serial); // use a software serial port to talk to the display

  genie.AttachEventHandler(myGenieEventHandler); // Attach the user function Event Handler for processing events
  // Reset the Display (change D4 to D2 if you have original 4D Arduino Adaptor)
  // THIS IS IMPORTANT AND CAN PREVENT OUT OF SYNC ISSUES, SLOW SPEED RESPONSE ETC
  // If NOT using a 4D Arduino Adaptor, digitalWrites must be reversed as Display Reset is Active Low, and
  // the 4D Arduino Adaptors invert this signal so must be Active High.
  pinMode(RESETLINE, OUTPUT);  // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(RESETLINE, 1);  // Reset the Display via D4
  delay(100);
  digitalWrite(RESETLINE, 0);  // unReset the Display via D4

  delay (5000); //let the display start up after the reset (This is important)
  Serial.println("Hello Arduino!");
}

void loop()
{
  static long waitPeriod = millis();

  genie.DoEvents(); // This calls the library each loop to process the queued responses from the display  

  if (millis() >= waitPeriod)
  {
    //do something here 
    waitPeriod = millis() + 50;                    // rerun this code in another 50ms time.  
  }
}

/////////////////////////////////////////////////////////////////////
//
// This is the user's event handler. It is called by genieDoEvents()
// when the following conditions are true
//
//		The link is in an IDLE state, and
//		There is an event to handle
//
// The event can be either a REPORT_EVENT frame sent asynchronously
// from the display or a REPORT_OBJ frame sent by the display in
// response to a READ_OBJ request.
//

// LONG HAND VERSION, MAYBE MORE VISIBLE AND MORE LIKE VERSION 1 OF THE LIBRARY
void myGenieEventHandler(void)
{
  genieFrame Event;
  genie.DequeueEvent(&Event);

  int ledDigits_val = 0;
  //If the cmd received is from a Reported Event
  if (Event.reportObject.cmd == GENIE_REPORT_EVENT)
  { 
    if (Event.reportObject.object == GENIE_OBJ_LED_DIGITS)    // If the Reported Message was from a LED digits object
    {
      if (Event.reportObject.index == 0)                      // If Leddigits0
      {
        ledDigits_val = genie.GetEventData(&Event);           // Receive the event data from Leddigits0
        Serial.print("Counter value: ");
        Serial.println(ledDigits_val);
      }
    }
  }

  //This can be expanded as more objects are added that need to be captured

  //Event.reportObject.cmd is used to determine the command of that event, such as an reported event, reported object
  //Event.reportObject.object is used to determine the object type, such as a LED digits object
  //Event.reportObject.index is used to determine the index of the object, such as Leddigits0
  //genie.GetEventData(&Event) us used to save the data from the Event, into a variable.
}

